% Silhouette Comparison Routines
%
% Sample usage for code implementing the turning angle metric 
% and chamfer distance on silhouettes.
%
% Created July 21, 2004
% Nicholas R. Howe
% 
% This code comes with no warranty for any purpose.
% Permission is granted to use for noncommercial purposes.

% Compile mex-files
mex 'chain.cpp'
mex 'unchain.cpp'
mex 'taCompare.cpp'
mex 'chamferDistance.cpp'
mex 'chamferView.cpp'

% Create some simple shapes
sil1 = ones(2,18);
sil2 = ones(6,6);
sil3 = [1 1 0 0; 1 1 0 0; 1 1 1 1; 1 1 1 1];

% The chain code starts from the first nonzero point.
% If you have silhouettes of a person, taking the 
% transpose will make this the top of the head.

sil1 = sil1';
sil2 = sil2';
sil3 = sil3';

% Convert to chain code representation.
% Shape must be a single 8-connected component.
ch1 = chain(sil1);
ch2 = chain(sil2);
ch3 = chain(sil3);

% Turning angle comparisons
taCompare(ch1,ch2)
taCompare({ch1,ch2,ch3},{ch1,ch2,ch3})

% Half chamfer distance comparisons
% To get full chamfer distance, apply
% symmetrically.  However, if you use the outline of
% the shapes instead of the silhouettes themselves,
% I have found that the half-chamfer distance works fine.
%
% Some notes:
% 1.  Computes half chamfer distance => use on outlines of silhouettes.
% 2.  The chained shape is assumed to be centered within the distance
%     transform matrix provided.
% 3.  Results inaccurate if chained silhouette extends beyond edge of
%     distance transform matrix.

% Get distance transforms
dt1 = chamferTransform(sil1);
dt2 = chamferTransform(sil2);
dt3 = chamferTransform(sil3);

% Do comparisons
chamferDistance(ch1,dt2)
chamferDistance(ch2,dt1)
chamferDistance({ch1,ch2,ch3},dt1)
chamferDistance({ch1,ch2,ch3},dt2)
chamferDistance({ch1,ch2,ch3},dt3)

% Scaled comparisons
chamferDistance(ch2,dt3)
chamferDistance(ch3,dt2)
chamferDistance(ch2,dt3,2/3)
chamferDistance(ch3,dt2,1.5)

% Visualization
% White crosses show points at which distance transform is summed
imagesc(dt3)
chv = chamferView(ch1,dt3);
[i,j] = find(~isnan(chv));
hold on
plot(j,i,'w+')
axis('image','xy')

imagesc(dt1)
chv = chamferView(ch2,dt1);
[i,j] = find(~isnan(chv));
hold on
plot(j,i,'w+')
axis('image','xy')
